#include "StdAfx.h"
#include ".\npsyncobject.h"


struct CNPCriticalSection::Context
{
	CRITICAL_SECTION m_cs;
};

CNPCriticalSection::CNPCriticalSection(DWORD dwSpinCount /*= 4000*/)
{
	m_pContext = new Context;

	::InitializeCriticalSectionAndSpinCount(&m_pContext->m_cs, dwSpinCount);
}

CNPCriticalSection::~CNPCriticalSection()
{
	if (NULL != m_pContext)
	{
		::DeleteCriticalSection(&m_pContext->m_cs);
		delete m_pContext;
	}
}

CNPCriticalSection::operator CRITICAL_SECTION*()
{
	return &m_pContext->m_cs;
}

void CNPCriticalSection::Attach(CRITICAL_SECTION* p)
{
	::DeleteCriticalSection(&m_pContext->m_cs);
	m_pContext->m_cs = *p;
}

CRITICAL_SECTION CNPCriticalSection::Detach()
{
	CRITICAL_SECTION cs = m_pContext->m_cs;
	::InitializeCriticalSection(&m_pContext->m_cs);
	return cs;
}

void CNPCriticalSection::Enter()
{
	::EnterCriticalSection(&m_pContext->m_cs);
}

BOOL CNPCriticalSection::TryEnter()
{
	return ::TryEnterCriticalSection(&m_pContext->m_cs);
}

void CNPCriticalSection::Leave()
{
	::LeaveCriticalSection(&m_pContext->m_cs);
}

BOOL CNPCriticalSection::Lock(DWORD /*dwMilliseconds = INFINITE*/)
{
	Enter(); 
	return TRUE;
}

BOOL CNPCriticalSection::Unlock()
{
	Leave(); 
	return TRUE;
}


struct CNPLock::Context
{
	INPSynchObject* m_pObj;
	BOOL m_bLocked;
	DWORD dwReserved;
};
#define thisCtx	((Context*)m_context)
CNPLock::CNPLock(INPSynchObject& r, DWORD dwMilliseconds /*= INFINITE*/)
{

	thisCtx->m_pObj = &r;
	thisCtx->m_bLocked = FALSE;

	Lock(dwMilliseconds);
}

CNPLock::CNPLock(INPSynchObject* p, DWORD dwMilliseconds /*= INFINITE*/)
{
	thisCtx->m_pObj = p;
	thisCtx->m_bLocked = FALSE;

	Lock(dwMilliseconds);
}

CNPLock::~CNPLock()
{
	Unlock();
}

BOOL CNPLock::Lock(DWORD dwMilliseconds /*= INFINITE*/)
{
	BOOL nRet = FALSE;

	if (NULL == thisCtx->m_pObj)
	{		
		return nRet;
	}

	nRet = thisCtx->m_pObj->Lock(dwMilliseconds);
	thisCtx->m_bLocked = nRet;	

	return nRet;
}

BOOL CNPLock::Unlock()
{
	BOOL nRet = FALSE;

	if (NULL == thisCtx->m_pObj)
	{
		return nRet;
	}

	if (thisCtx->m_bLocked)
	{
		nRet = thisCtx->m_pObj->Unlock();
		thisCtx->m_bLocked = nRet;			
	}

	return nRet;
}

BOOL CNPLock::IsLocked()
{
	return thisCtx->m_bLocked;
}

void CNPLock::Attach(INPSynchObject* pObj)
{
	thisCtx->m_pObj = pObj;
	thisCtx->m_bLocked = (NULL != thisCtx->m_pObj);
}

INPSynchObject* CNPLock::Detach()
{
	INPSynchObject* pObj = thisCtx->m_pObj;
	thisCtx->m_pObj = NULL;
	thisCtx->m_bLocked = FALSE;
	return pObj;
}
